# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.zero_trust.dex import (
    traceroute_test_get_params,
    traceroute_test_percentiles_params,
    traceroute_test_network_path_params,
)
from ....types.zero_trust.dex.traceroute import Traceroute
from ....types.zero_trust.network_path_response import NetworkPathResponse
from ....types.zero_trust.dex.traceroute_test_percentiles_response import TracerouteTestPercentilesResponse

__all__ = ["TracerouteTestsResource", "AsyncTracerouteTestsResource"]


class TracerouteTestsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TracerouteTestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TracerouteTestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TracerouteTestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TracerouteTestsResourceWithStreamingResponse(self)

    def get(
        self,
        test_id: str,
        *,
        account_id: str,
        from_: str,
        interval: Literal["minute", "hour"],
        to: str,
        colo: str | Omit = omit,
        device_id: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Traceroute]:
        """
        Get test details and aggregate performance metrics for an traceroute test for a
        given time period between 1 hour and 7 days.

        Args:
          test_id: API Resource UUID tag.

          from_: Start time for aggregate metrics in ISO ms

          interval: Time interval for aggregate time slots.

          to: End time for aggregate metrics in ISO ms

          colo: Optionally filter result stats to a Cloudflare colo. Cannot be used in
              combination with deviceId param.

          device_id: Optionally filter result stats to a specific device(s). Cannot be used in
              combination with colo param.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dex/traceroute-tests/{test_id}", account_id=account_id, test_id=test_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "interval": interval,
                        "to": to,
                        "colo": colo,
                        "device_id": device_id,
                    },
                    traceroute_test_get_params.TracerouteTestGetParams,
                ),
                post_parser=ResultWrapper[Optional[Traceroute]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Traceroute]], ResultWrapper[Traceroute]),
        )

    def network_path(
        self,
        test_id: str,
        *,
        account_id: str,
        device_id: str,
        from_: str,
        interval: Literal["minute", "hour"],
        to: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NetworkPathResponse]:
        """
        Get a breakdown of metrics by hop for individual traceroute test runs

        Args:
          test_id: API Resource UUID tag.

          device_id: Device to filter tracroute result runs to

          from_: Start time for aggregate metrics in ISO ms

          interval: Time interval for aggregate time slots.

          to: End time for aggregate metrics in ISO ms

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dex/traceroute-tests/{test_id}/network-path",
                account_id=account_id,
                test_id=test_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "device_id": device_id,
                        "from_": from_,
                        "interval": interval,
                        "to": to,
                    },
                    traceroute_test_network_path_params.TracerouteTestNetworkPathParams,
                ),
                post_parser=ResultWrapper[Optional[NetworkPathResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NetworkPathResponse]], ResultWrapper[NetworkPathResponse]),
        )

    def percentiles(
        self,
        test_id: str,
        *,
        account_id: str,
        from_: str,
        to: str,
        colo: str | Omit = omit,
        device_id: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TracerouteTestPercentilesResponse]:
        """
        Get percentiles for a traceroute test for a given time period between 1 hour and
        7 days.

        Args:
          test_id: API Resource UUID tag.

          from_: Start time for the query in ISO (RFC3339 - ISO 8601) format

          to: End time for the query in ISO (RFC3339 - ISO 8601) format

          colo: Optionally filter result stats to a Cloudflare colo. Cannot be used in
              combination with deviceId param.

          device_id: Optionally filter result stats to a specific device(s). Cannot be used in
              combination with colo param.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dex/traceroute-tests/{test_id}/percentiles",
                account_id=account_id,
                test_id=test_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "colo": colo,
                        "device_id": device_id,
                    },
                    traceroute_test_percentiles_params.TracerouteTestPercentilesParams,
                ),
                post_parser=ResultWrapper[Optional[TracerouteTestPercentilesResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TracerouteTestPercentilesResponse]], ResultWrapper[TracerouteTestPercentilesResponse]
            ),
        )


class AsyncTracerouteTestsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTracerouteTestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTracerouteTestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTracerouteTestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTracerouteTestsResourceWithStreamingResponse(self)

    async def get(
        self,
        test_id: str,
        *,
        account_id: str,
        from_: str,
        interval: Literal["minute", "hour"],
        to: str,
        colo: str | Omit = omit,
        device_id: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Traceroute]:
        """
        Get test details and aggregate performance metrics for an traceroute test for a
        given time period between 1 hour and 7 days.

        Args:
          test_id: API Resource UUID tag.

          from_: Start time for aggregate metrics in ISO ms

          interval: Time interval for aggregate time slots.

          to: End time for aggregate metrics in ISO ms

          colo: Optionally filter result stats to a Cloudflare colo. Cannot be used in
              combination with deviceId param.

          device_id: Optionally filter result stats to a specific device(s). Cannot be used in
              combination with colo param.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dex/traceroute-tests/{test_id}", account_id=account_id, test_id=test_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_": from_,
                        "interval": interval,
                        "to": to,
                        "colo": colo,
                        "device_id": device_id,
                    },
                    traceroute_test_get_params.TracerouteTestGetParams,
                ),
                post_parser=ResultWrapper[Optional[Traceroute]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Traceroute]], ResultWrapper[Traceroute]),
        )

    async def network_path(
        self,
        test_id: str,
        *,
        account_id: str,
        device_id: str,
        from_: str,
        interval: Literal["minute", "hour"],
        to: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[NetworkPathResponse]:
        """
        Get a breakdown of metrics by hop for individual traceroute test runs

        Args:
          test_id: API Resource UUID tag.

          device_id: Device to filter tracroute result runs to

          from_: Start time for aggregate metrics in ISO ms

          interval: Time interval for aggregate time slots.

          to: End time for aggregate metrics in ISO ms

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dex/traceroute-tests/{test_id}/network-path",
                account_id=account_id,
                test_id=test_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "device_id": device_id,
                        "from_": from_,
                        "interval": interval,
                        "to": to,
                    },
                    traceroute_test_network_path_params.TracerouteTestNetworkPathParams,
                ),
                post_parser=ResultWrapper[Optional[NetworkPathResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[NetworkPathResponse]], ResultWrapper[NetworkPathResponse]),
        )

    async def percentiles(
        self,
        test_id: str,
        *,
        account_id: str,
        from_: str,
        to: str,
        colo: str | Omit = omit,
        device_id: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TracerouteTestPercentilesResponse]:
        """
        Get percentiles for a traceroute test for a given time period between 1 hour and
        7 days.

        Args:
          test_id: API Resource UUID tag.

          from_: Start time for the query in ISO (RFC3339 - ISO 8601) format

          to: End time for the query in ISO (RFC3339 - ISO 8601) format

          colo: Optionally filter result stats to a Cloudflare colo. Cannot be used in
              combination with deviceId param.

          device_id: Optionally filter result stats to a specific device(s). Cannot be used in
              combination with colo param.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not test_id:
            raise ValueError(f"Expected a non-empty value for `test_id` but received {test_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dex/traceroute-tests/{test_id}/percentiles",
                account_id=account_id,
                test_id=test_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_": from_,
                        "to": to,
                        "colo": colo,
                        "device_id": device_id,
                    },
                    traceroute_test_percentiles_params.TracerouteTestPercentilesParams,
                ),
                post_parser=ResultWrapper[Optional[TracerouteTestPercentilesResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TracerouteTestPercentilesResponse]], ResultWrapper[TracerouteTestPercentilesResponse]
            ),
        )


class TracerouteTestsResourceWithRawResponse:
    def __init__(self, traceroute_tests: TracerouteTestsResource) -> None:
        self._traceroute_tests = traceroute_tests

        self.get = to_raw_response_wrapper(
            traceroute_tests.get,
        )
        self.network_path = to_raw_response_wrapper(
            traceroute_tests.network_path,
        )
        self.percentiles = to_raw_response_wrapper(
            traceroute_tests.percentiles,
        )


class AsyncTracerouteTestsResourceWithRawResponse:
    def __init__(self, traceroute_tests: AsyncTracerouteTestsResource) -> None:
        self._traceroute_tests = traceroute_tests

        self.get = async_to_raw_response_wrapper(
            traceroute_tests.get,
        )
        self.network_path = async_to_raw_response_wrapper(
            traceroute_tests.network_path,
        )
        self.percentiles = async_to_raw_response_wrapper(
            traceroute_tests.percentiles,
        )


class TracerouteTestsResourceWithStreamingResponse:
    def __init__(self, traceroute_tests: TracerouteTestsResource) -> None:
        self._traceroute_tests = traceroute_tests

        self.get = to_streamed_response_wrapper(
            traceroute_tests.get,
        )
        self.network_path = to_streamed_response_wrapper(
            traceroute_tests.network_path,
        )
        self.percentiles = to_streamed_response_wrapper(
            traceroute_tests.percentiles,
        )


class AsyncTracerouteTestsResourceWithStreamingResponse:
    def __init__(self, traceroute_tests: AsyncTracerouteTestsResource) -> None:
        self._traceroute_tests = traceroute_tests

        self.get = async_to_streamed_response_wrapper(
            traceroute_tests.get,
        )
        self.network_path = async_to_streamed_response_wrapper(
            traceroute_tests.network_path,
        )
        self.percentiles = async_to_streamed_response_wrapper(
            traceroute_tests.percentiles,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.page_rules import (
    page_rule_edit_params,
    page_rule_list_params,
    page_rule_create_params,
    page_rule_update_params,
)
from ...types.page_rules.page_rule import PageRule
from ...types.page_rules.target_param import TargetParam
from ...types.page_rules.page_rule_list_response import PageRuleListResponse
from ...types.page_rules.page_rule_delete_response import PageRuleDeleteResponse

__all__ = ["PageRulesResource", "AsyncPageRulesResource"]


class PageRulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PageRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PageRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PageRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PageRulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        actions: Iterable[page_rule_create_params.Action],
        targets: Iterable[TargetParam],
        priority: int | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """
        Creates a new Page Rule.

        Args:
          zone_id: Identifier.

          actions: The set of actions to perform if the targets of this rule match the request.
              Actions can redirect to another URL or override settings, but not both.

          targets: The rule targets to evaluate on each request.

          priority: The priority of the rule, used to define which Page Rule is processed over
              another. A higher number indicates a higher priority. For example, if you have a
              catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
              take precedence (rule B: `/images/special/*`), specify a higher priority for
              rule B so it overrides rule A.

          status: The status of the Page Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/pagerules", zone_id=zone_id),
            body=maybe_transform(
                {
                    "actions": actions,
                    "targets": targets,
                    "priority": priority,
                    "status": status,
                },
                page_rule_create_params.PageRuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )

    def update(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        actions: Iterable[page_rule_update_params.Action],
        targets: Iterable[TargetParam],
        priority: int | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """Replaces the configuration of an existing Page Rule.

        The configuration of the
        updated Page Rule will exactly match the data passed in the API request.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          actions: The set of actions to perform if the targets of this rule match the request.
              Actions can redirect to another URL or override settings, but not both.

          targets: The rule targets to evaluate on each request.

          priority: The priority of the rule, used to define which Page Rule is processed over
              another. A higher number indicates a higher priority. For example, if you have a
              catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
              take precedence (rule B: `/images/special/*`), specify a higher priority for
              rule B so it overrides rule A.

          status: The status of the Page Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            body=maybe_transform(
                {
                    "actions": actions,
                    "targets": targets,
                    "priority": priority,
                    "status": status,
                },
                page_rule_update_params.PageRuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )

    def list(
        self,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        order: Literal["status", "priority"] | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRuleListResponse]:
        """
        Fetches Page Rules in a zone.

        Args:
          zone_id: Identifier.

          direction: The direction used to sort returned Page Rules.

          match: When set to `all`, all the search requirements must match. When set to `any`,
              only one of the search requirements has to match.

          order: The field used to sort returned Page Rules.

          status: The status of the Page Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/pagerules", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "order": order,
                        "status": status,
                    },
                    page_rule_list_params.PageRuleListParams,
                ),
                post_parser=ResultWrapper[Optional[PageRuleListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRuleListResponse]], ResultWrapper[PageRuleListResponse]),
        )

    def delete(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRuleDeleteResponse]:
        """
        Deletes an existing Page Rule.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRuleDeleteResponse]], ResultWrapper[PageRuleDeleteResponse]),
        )

    def edit(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        actions: Iterable[page_rule_edit_params.Action] | Omit = omit,
        priority: int | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        targets: Iterable[TargetParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """
        Updates one or more fields of an existing Page Rule.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          actions: The set of actions to perform if the targets of this rule match the request.
              Actions can redirect to another URL or override settings, but not both.

          priority: The priority of the rule, used to define which Page Rule is processed over
              another. A higher number indicates a higher priority. For example, if you have a
              catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
              take precedence (rule B: `/images/special/*`), specify a higher priority for
              rule B so it overrides rule A.

          status: The status of the Page Rule.

          targets: The rule targets to evaluate on each request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            body=maybe_transform(
                {
                    "actions": actions,
                    "priority": priority,
                    "status": status,
                    "targets": targets,
                },
                page_rule_edit_params.PageRuleEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )

    def get(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """
        Fetches the details of a Page Rule.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )


class AsyncPageRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPageRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPageRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPageRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPageRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        actions: Iterable[page_rule_create_params.Action],
        targets: Iterable[TargetParam],
        priority: int | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """
        Creates a new Page Rule.

        Args:
          zone_id: Identifier.

          actions: The set of actions to perform if the targets of this rule match the request.
              Actions can redirect to another URL or override settings, but not both.

          targets: The rule targets to evaluate on each request.

          priority: The priority of the rule, used to define which Page Rule is processed over
              another. A higher number indicates a higher priority. For example, if you have a
              catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
              take precedence (rule B: `/images/special/*`), specify a higher priority for
              rule B so it overrides rule A.

          status: The status of the Page Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/pagerules", zone_id=zone_id),
            body=await async_maybe_transform(
                {
                    "actions": actions,
                    "targets": targets,
                    "priority": priority,
                    "status": status,
                },
                page_rule_create_params.PageRuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )

    async def update(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        actions: Iterable[page_rule_update_params.Action],
        targets: Iterable[TargetParam],
        priority: int | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """Replaces the configuration of an existing Page Rule.

        The configuration of the
        updated Page Rule will exactly match the data passed in the API request.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          actions: The set of actions to perform if the targets of this rule match the request.
              Actions can redirect to another URL or override settings, but not both.

          targets: The rule targets to evaluate on each request.

          priority: The priority of the rule, used to define which Page Rule is processed over
              another. A higher number indicates a higher priority. For example, if you have a
              catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
              take precedence (rule B: `/images/special/*`), specify a higher priority for
              rule B so it overrides rule A.

          status: The status of the Page Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            body=await async_maybe_transform(
                {
                    "actions": actions,
                    "targets": targets,
                    "priority": priority,
                    "status": status,
                },
                page_rule_update_params.PageRuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )

    async def list(
        self,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        match: Literal["any", "all"] | Omit = omit,
        order: Literal["status", "priority"] | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRuleListResponse]:
        """
        Fetches Page Rules in a zone.

        Args:
          zone_id: Identifier.

          direction: The direction used to sort returned Page Rules.

          match: When set to `all`, all the search requirements must match. When set to `any`,
              only one of the search requirements has to match.

          order: The field used to sort returned Page Rules.

          status: The status of the Page Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/pagerules", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "direction": direction,
                        "match": match,
                        "order": order,
                        "status": status,
                    },
                    page_rule_list_params.PageRuleListParams,
                ),
                post_parser=ResultWrapper[Optional[PageRuleListResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRuleListResponse]], ResultWrapper[PageRuleListResponse]),
        )

    async def delete(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRuleDeleteResponse]:
        """
        Deletes an existing Page Rule.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRuleDeleteResponse]], ResultWrapper[PageRuleDeleteResponse]),
        )

    async def edit(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        actions: Iterable[page_rule_edit_params.Action] | Omit = omit,
        priority: int | Omit = omit,
        status: Literal["active", "disabled"] | Omit = omit,
        targets: Iterable[TargetParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """
        Updates one or more fields of an existing Page Rule.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          actions: The set of actions to perform if the targets of this rule match the request.
              Actions can redirect to another URL or override settings, but not both.

          priority: The priority of the rule, used to define which Page Rule is processed over
              another. A higher number indicates a higher priority. For example, if you have a
              catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
              take precedence (rule B: `/images/special/*`), specify a higher priority for
              rule B so it overrides rule A.

          status: The status of the Page Rule.

          targets: The rule targets to evaluate on each request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            body=await async_maybe_transform(
                {
                    "actions": actions,
                    "priority": priority,
                    "status": status,
                    "targets": targets,
                },
                page_rule_edit_params.PageRuleEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )

    async def get(
        self,
        pagerule_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PageRule]:
        """
        Fetches the details of a Page Rule.

        Args:
          zone_id: Identifier.

          pagerule_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not pagerule_id:
            raise ValueError(f"Expected a non-empty value for `pagerule_id` but received {pagerule_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/pagerules/{pagerule_id}", zone_id=zone_id, pagerule_id=pagerule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PageRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PageRule]], ResultWrapper[PageRule]),
        )


class PageRulesResourceWithRawResponse:
    def __init__(self, page_rules: PageRulesResource) -> None:
        self._page_rules = page_rules

        self.create = to_raw_response_wrapper(
            page_rules.create,
        )
        self.update = to_raw_response_wrapper(
            page_rules.update,
        )
        self.list = to_raw_response_wrapper(
            page_rules.list,
        )
        self.delete = to_raw_response_wrapper(
            page_rules.delete,
        )
        self.edit = to_raw_response_wrapper(
            page_rules.edit,
        )
        self.get = to_raw_response_wrapper(
            page_rules.get,
        )


class AsyncPageRulesResourceWithRawResponse:
    def __init__(self, page_rules: AsyncPageRulesResource) -> None:
        self._page_rules = page_rules

        self.create = async_to_raw_response_wrapper(
            page_rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            page_rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            page_rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            page_rules.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            page_rules.edit,
        )
        self.get = async_to_raw_response_wrapper(
            page_rules.get,
        )


class PageRulesResourceWithStreamingResponse:
    def __init__(self, page_rules: PageRulesResource) -> None:
        self._page_rules = page_rules

        self.create = to_streamed_response_wrapper(
            page_rules.create,
        )
        self.update = to_streamed_response_wrapper(
            page_rules.update,
        )
        self.list = to_streamed_response_wrapper(
            page_rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            page_rules.delete,
        )
        self.edit = to_streamed_response_wrapper(
            page_rules.edit,
        )
        self.get = to_streamed_response_wrapper(
            page_rules.get,
        )


class AsyncPageRulesResourceWithStreamingResponse:
    def __init__(self, page_rules: AsyncPageRulesResource) -> None:
        self._page_rules = page_rules

        self.create = async_to_streamed_response_wrapper(
            page_rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            page_rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            page_rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            page_rules.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            page_rules.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            page_rules.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.quality import iqi_summary_params, iqi_timeseries_groups_params
from ....types.radar.quality.iqi_summary_response import IQISummaryResponse
from ....types.radar.quality.iqi_timeseries_groups_response import IQITimeseriesGroupsResponse

__all__ = ["IQIResource", "AsyncIQIResource"]


class IQIResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IQIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IQIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IQIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IQIResourceWithStreamingResponse(self)

    def summary(
        self,
        *,
        metric: Literal["BANDWIDTH", "DNS", "LATENCY"],
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IQISummaryResponse:
        """
        Retrieves a summary (percentiles) of bandwidth, latency, or DNS response time
        from the Radar Internet Quality Index (IQI).

        Args:
          metric: Defines which metric to return (bandwidth, latency, or DNS response time).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/quality/iqi/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "metric": metric,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "name": name,
                    },
                    iqi_summary_params.IQISummaryParams,
                ),
                post_parser=ResultWrapper[IQISummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[IQISummaryResponse], ResultWrapper[IQISummaryResponse]),
        )

    def timeseries_groups(
        self,
        *,
        metric: Literal["BANDWIDTH", "DNS", "LATENCY"],
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        interpolation: bool | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IQITimeseriesGroupsResponse:
        """
        Retrieves a time series (percentiles) of bandwidth, latency, or DNS response
        time from the Radar Internet Quality Index (IQI).

        Args:
          metric: Defines which metric to return (bandwidth, latency, or DNS response time).

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          interpolation: Enables interpolation for all series (using the average).

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/quality/iqi/timeseries_groups",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "metric": metric,
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "interpolation": interpolation,
                        "location": location,
                        "name": name,
                    },
                    iqi_timeseries_groups_params.IQITimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[IQITimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[IQITimeseriesGroupsResponse], ResultWrapper[IQITimeseriesGroupsResponse]),
        )


class AsyncIQIResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIQIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIQIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIQIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIQIResourceWithStreamingResponse(self)

    async def summary(
        self,
        *,
        metric: Literal["BANDWIDTH", "DNS", "LATENCY"],
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IQISummaryResponse:
        """
        Retrieves a summary (percentiles) of bandwidth, latency, or DNS response time
        from the Radar Internet Quality Index (IQI).

        Args:
          metric: Defines which metric to return (bandwidth, latency, or DNS response time).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/quality/iqi/summary",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "metric": metric,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "location": location,
                        "name": name,
                    },
                    iqi_summary_params.IQISummaryParams,
                ),
                post_parser=ResultWrapper[IQISummaryResponse]._unwrapper,
            ),
            cast_to=cast(Type[IQISummaryResponse], ResultWrapper[IQISummaryResponse]),
        )

    async def timeseries_groups(
        self,
        *,
        metric: Literal["BANDWIDTH", "DNS", "LATENCY"],
        agg_interval: Literal["15m", "1h", "1d", "1w"] | Omit = omit,
        asn: SequenceNotStr[str] | Omit = omit,
        continent: SequenceNotStr[str] | Omit = omit,
        date_end: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        date_range: SequenceNotStr[str] | Omit = omit,
        date_start: SequenceNotStr[Union[str, datetime]] | Omit = omit,
        format: Literal["JSON", "CSV"] | Omit = omit,
        interpolation: bool | Omit = omit,
        location: SequenceNotStr[str] | Omit = omit,
        name: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IQITimeseriesGroupsResponse:
        """
        Retrieves a time series (percentiles) of bandwidth, latency, or DNS response
        time from the Radar Internet Quality Index (IQI).

        Args:
          metric: Defines which metric to return (bandwidth, latency, or DNS response time).

          agg_interval: Aggregation interval of the results (e.g., in 15 minutes or 1 hour intervals).
              Refer to
              [Aggregation intervals](https://developers.cloudflare.com/radar/concepts/aggregation-intervals/).

          asn: Filters results by Autonomous System. Specify one or more Autonomous System
              Numbers (ASNs) as a comma-separated list. Prefix with `-` to exclude ASNs from
              results. For example, `-174, 3356` excludes results from AS174, but includes
              results from AS3356.

          continent: Filters results by continent. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude continents from results. For example, `-EU,NA`
              excludes results from EU, but includes results from NA.

          date_end: End of the date range (inclusive).

          date_range: Filters results by date range. For example, use `7d` and `7dcontrol` to compare
              this week with the previous week. Use this parameter or set specific start and
              end dates (`dateStart` and `dateEnd` parameters).

          date_start: Start of the date range.

          format: Format in which results will be returned.

          interpolation: Enables interpolation for all series (using the average).

          location: Filters results by location. Specify a comma-separated list of alpha-2 codes.
              Prefix with `-` to exclude locations from results. For example, `-US,PT`
              excludes results from the US, but includes results from PT.

          name: Array of names used to label the series in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/quality/iqi/timeseries_groups",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "metric": metric,
                        "agg_interval": agg_interval,
                        "asn": asn,
                        "continent": continent,
                        "date_end": date_end,
                        "date_range": date_range,
                        "date_start": date_start,
                        "format": format,
                        "interpolation": interpolation,
                        "location": location,
                        "name": name,
                    },
                    iqi_timeseries_groups_params.IQITimeseriesGroupsParams,
                ),
                post_parser=ResultWrapper[IQITimeseriesGroupsResponse]._unwrapper,
            ),
            cast_to=cast(Type[IQITimeseriesGroupsResponse], ResultWrapper[IQITimeseriesGroupsResponse]),
        )


class IQIResourceWithRawResponse:
    def __init__(self, iqi: IQIResource) -> None:
        self._iqi = iqi

        self.summary = to_raw_response_wrapper(
            iqi.summary,
        )
        self.timeseries_groups = to_raw_response_wrapper(
            iqi.timeseries_groups,
        )


class AsyncIQIResourceWithRawResponse:
    def __init__(self, iqi: AsyncIQIResource) -> None:
        self._iqi = iqi

        self.summary = async_to_raw_response_wrapper(
            iqi.summary,
        )
        self.timeseries_groups = async_to_raw_response_wrapper(
            iqi.timeseries_groups,
        )


class IQIResourceWithStreamingResponse:
    def __init__(self, iqi: IQIResource) -> None:
        self._iqi = iqi

        self.summary = to_streamed_response_wrapper(
            iqi.summary,
        )
        self.timeseries_groups = to_streamed_response_wrapper(
            iqi.timeseries_groups,
        )


class AsyncIQIResourceWithStreamingResponse:
    def __init__(self, iqi: AsyncIQIResource) -> None:
        self._iqi = iqi

        self.summary = async_to_streamed_response_wrapper(
            iqi.summary,
        )
        self.timeseries_groups = async_to_streamed_response_wrapper(
            iqi.timeseries_groups,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["HistoryListParams"]


class HistoryListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    limit: int
    """Maximum amount of results to list. Default value is 10."""

    offset: int
    """Ordinal number to start listing the results with. Default value is 0."""

    sort_field: Annotated[
        Literal["id", "user_id", "description", "created_at", "updated_at"], PropertyInfo(alias="sortField")
    ]
    """The field to sort by. Default is updated_at."""

    sort_order: Annotated[Literal["DESC", "ASC"], PropertyInfo(alias="sortOrder")]
    """Sorting order. Default is DESC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.realtime_kit import active_session_create_poll_params, active_session_kick_participants_params
from ...types.realtime_kit.active_session_create_poll_response import ActiveSessionCreatePollResponse
from ...types.realtime_kit.active_session_kick_participants_response import ActiveSessionKickParticipantsResponse
from ...types.realtime_kit.active_session_get_active_session_response import ActiveSessionGetActiveSessionResponse
from ...types.realtime_kit.active_session_kick_all_participants_response import ActiveSessionKickAllParticipantsResponse

__all__ = ["ActiveSessionResource", "AsyncActiveSessionResource"]


class ActiveSessionResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActiveSessionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ActiveSessionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActiveSessionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ActiveSessionResourceWithStreamingResponse(self)

    def create_poll(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        options: SequenceNotStr[str],
        question: str,
        anonymous: bool | Omit = omit,
        hide_votes: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionCreatePollResponse:
        """
        Creates a new poll in an active session for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          options: Different options for the question

          question: Question of the poll

          anonymous: if voters on a poll are anonymous

          hide_votes: if votes on an option are visible before a person votes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session/poll",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=maybe_transform(
                {
                    "options": options,
                    "question": question,
                    "anonymous": anonymous,
                    "hide_votes": hide_votes,
                },
                active_session_create_poll_params.ActiveSessionCreatePollParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionCreatePollResponse,
        )

    def get_active_session(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionGetActiveSessionResponse:
        """
        Returns details of an ongoing active session for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionGetActiveSessionResponse,
        )

    def kick_all_participants(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionKickAllParticipantsResponse:
        """
        Kicks all participants from an active session for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session/kick-all",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionKickAllParticipantsResponse,
        )

    def kick_participants(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        custom_participant_ids: SequenceNotStr[str],
        participant_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionKickParticipantsResponse:
        """
        Kicks one or more participants from an active session using user ID or custom
        participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session/kick",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=maybe_transform(
                {
                    "custom_participant_ids": custom_participant_ids,
                    "participant_ids": participant_ids,
                },
                active_session_kick_participants_params.ActiveSessionKickParticipantsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionKickParticipantsResponse,
        )


class AsyncActiveSessionResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActiveSessionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActiveSessionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActiveSessionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncActiveSessionResourceWithStreamingResponse(self)

    async def create_poll(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        options: SequenceNotStr[str],
        question: str,
        anonymous: bool | Omit = omit,
        hide_votes: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionCreatePollResponse:
        """
        Creates a new poll in an active session for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          options: Different options for the question

          question: Question of the poll

          anonymous: if voters on a poll are anonymous

          hide_votes: if votes on an option are visible before a person votes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session/poll",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=await async_maybe_transform(
                {
                    "options": options,
                    "question": question,
                    "anonymous": anonymous,
                    "hide_votes": hide_votes,
                },
                active_session_create_poll_params.ActiveSessionCreatePollParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionCreatePollResponse,
        )

    async def get_active_session(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionGetActiveSessionResponse:
        """
        Returns details of an ongoing active session for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionGetActiveSessionResponse,
        )

    async def kick_all_participants(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionKickAllParticipantsResponse:
        """
        Kicks all participants from an active session for the given meeting ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session/kick-all",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionKickAllParticipantsResponse,
        )

    async def kick_participants(
        self,
        meeting_id: str,
        *,
        account_id: str,
        app_id: str,
        custom_participant_ids: SequenceNotStr[str],
        participant_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActiveSessionKickParticipantsResponse:
        """
        Kicks one or more participants from an active session using user ID or custom
        participant ID.

        Args:
          account_id: The account identifier tag.

          app_id: The app identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not meeting_id:
            raise ValueError(f"Expected a non-empty value for `meeting_id` but received {meeting_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/realtime/kit/{app_id}/meetings/{meeting_id}/active-session/kick",
                account_id=account_id,
                app_id=app_id,
                meeting_id=meeting_id,
            ),
            body=await async_maybe_transform(
                {
                    "custom_participant_ids": custom_participant_ids,
                    "participant_ids": participant_ids,
                },
                active_session_kick_participants_params.ActiveSessionKickParticipantsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActiveSessionKickParticipantsResponse,
        )


class ActiveSessionResourceWithRawResponse:
    def __init__(self, active_session: ActiveSessionResource) -> None:
        self._active_session = active_session

        self.create_poll = to_raw_response_wrapper(
            active_session.create_poll,
        )
        self.get_active_session = to_raw_response_wrapper(
            active_session.get_active_session,
        )
        self.kick_all_participants = to_raw_response_wrapper(
            active_session.kick_all_participants,
        )
        self.kick_participants = to_raw_response_wrapper(
            active_session.kick_participants,
        )


class AsyncActiveSessionResourceWithRawResponse:
    def __init__(self, active_session: AsyncActiveSessionResource) -> None:
        self._active_session = active_session

        self.create_poll = async_to_raw_response_wrapper(
            active_session.create_poll,
        )
        self.get_active_session = async_to_raw_response_wrapper(
            active_session.get_active_session,
        )
        self.kick_all_participants = async_to_raw_response_wrapper(
            active_session.kick_all_participants,
        )
        self.kick_participants = async_to_raw_response_wrapper(
            active_session.kick_participants,
        )


class ActiveSessionResourceWithStreamingResponse:
    def __init__(self, active_session: ActiveSessionResource) -> None:
        self._active_session = active_session

        self.create_poll = to_streamed_response_wrapper(
            active_session.create_poll,
        )
        self.get_active_session = to_streamed_response_wrapper(
            active_session.get_active_session,
        )
        self.kick_all_participants = to_streamed_response_wrapper(
            active_session.kick_all_participants,
        )
        self.kick_participants = to_streamed_response_wrapper(
            active_session.kick_participants,
        )


class AsyncActiveSessionResourceWithStreamingResponse:
    def __init__(self, active_session: AsyncActiveSessionResource) -> None:
        self._active_session = active_session

        self.create_poll = async_to_streamed_response_wrapper(
            active_session.create_poll,
        )
        self.get_active_session = async_to_streamed_response_wrapper(
            active_session.get_active_session,
        )
        self.kick_all_participants = async_to_streamed_response_wrapper(
            active_session.kick_all_participants,
        )
        self.kick_participants = async_to_streamed_response_wrapper(
            active_session.kick_participants,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tests import Tests as Tests
from .unique_devices import UniqueDevices as UniqueDevices
from .unique_device_list_params import UniqueDeviceListParams as UniqueDeviceListParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Union, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust import tunnel_list_params
from .cloudflared.cloudflared import (
    CloudflaredResource,
    AsyncCloudflaredResource,
    CloudflaredResourceWithRawResponse,
    AsyncCloudflaredResourceWithRawResponse,
    CloudflaredResourceWithStreamingResponse,
    AsyncCloudflaredResourceWithStreamingResponse,
)
from .warp_connector.warp_connector import (
    WARPConnectorResource,
    AsyncWARPConnectorResource,
    WARPConnectorResourceWithRawResponse,
    AsyncWARPConnectorResourceWithRawResponse,
    WARPConnectorResourceWithStreamingResponse,
    AsyncWARPConnectorResourceWithStreamingResponse,
)
from ....types.zero_trust.tunnel_list_response import TunnelListResponse

__all__ = ["TunnelsResource", "AsyncTunnelsResource"]


class TunnelsResource(SyncAPIResource):
    @cached_property
    def cloudflared(self) -> CloudflaredResource:
        return CloudflaredResource(self._client)

    @cached_property
    def warp_connector(self) -> WARPConnectorResource:
        return WARPConnectorResource(self._client)

    @cached_property
    def with_raw_response(self) -> TunnelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TunnelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TunnelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TunnelsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        exclude_prefix: str | Omit = omit,
        existed_at: str | Omit = omit,
        include_prefix: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["inactive", "degraded", "healthy", "down"] | Omit = omit,
        tun_types: List[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]] | Omit = omit,
        uuid: str | Omit = omit,
        was_active_at: Union[str, datetime] | Omit = omit,
        was_inactive_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[TunnelListResponse]:
        """
        Lists and filters all types of Tunnels in an account.

        Args:
          account_id: Cloudflare account ID

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_deleted: If `true`, only include deleted tunnels. If `false`, exclude deleted tunnels. If
              empty, all tunnels will be included.

          name: A user-friendly name for the tunnel.

          page: Page number of paginated results.

          per_page: Number of results to display.

          status: The status of the tunnel. Valid values are `inactive` (tunnel has never been
              run), `degraded` (tunnel is active and able to serve traffic but in an unhealthy
              state), `healthy` (tunnel is active and able to serve traffic), or `down`
              (tunnel can not serve traffic as it has no connections to the Cloudflare Edge).

          tun_types: The types of tunnels to filter by, separated by commas.

          uuid: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tunnels", account_id=account_id),
            page=SyncV4PagePaginationArray[TunnelListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "exclude_prefix": exclude_prefix,
                        "existed_at": existed_at,
                        "include_prefix": include_prefix,
                        "is_deleted": is_deleted,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                        "tun_types": tun_types,
                        "uuid": uuid,
                        "was_active_at": was_active_at,
                        "was_inactive_at": was_inactive_at,
                    },
                    tunnel_list_params.TunnelListParams,
                ),
            ),
            model=cast(Any, TunnelListResponse),  # Union types cannot be passed in as arguments in the type system
        )


class AsyncTunnelsResource(AsyncAPIResource):
    @cached_property
    def cloudflared(self) -> AsyncCloudflaredResource:
        return AsyncCloudflaredResource(self._client)

    @cached_property
    def warp_connector(self) -> AsyncWARPConnectorResource:
        return AsyncWARPConnectorResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTunnelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTunnelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTunnelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTunnelsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        exclude_prefix: str | Omit = omit,
        existed_at: str | Omit = omit,
        include_prefix: str | Omit = omit,
        is_deleted: bool | Omit = omit,
        name: str | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        status: Literal["inactive", "degraded", "healthy", "down"] | Omit = omit,
        tun_types: List[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]] | Omit = omit,
        uuid: str | Omit = omit,
        was_active_at: Union[str, datetime] | Omit = omit,
        was_inactive_at: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TunnelListResponse, AsyncV4PagePaginationArray[TunnelListResponse]]:
        """
        Lists and filters all types of Tunnels in an account.

        Args:
          account_id: Cloudflare account ID

          existed_at: If provided, include only resources that were created (and not deleted) before
              this time. URL encoded.

          is_deleted: If `true`, only include deleted tunnels. If `false`, exclude deleted tunnels. If
              empty, all tunnels will be included.

          name: A user-friendly name for the tunnel.

          page: Page number of paginated results.

          per_page: Number of results to display.

          status: The status of the tunnel. Valid values are `inactive` (tunnel has never been
              run), `degraded` (tunnel is active and able to serve traffic but in an unhealthy
              state), `healthy` (tunnel is active and able to serve traffic), or `down`
              (tunnel can not serve traffic as it has no connections to the Cloudflare Edge).

          tun_types: The types of tunnels to filter by, separated by commas.

          uuid: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/tunnels", account_id=account_id),
            page=AsyncV4PagePaginationArray[TunnelListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "exclude_prefix": exclude_prefix,
                        "existed_at": existed_at,
                        "include_prefix": include_prefix,
                        "is_deleted": is_deleted,
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                        "tun_types": tun_types,
                        "uuid": uuid,
                        "was_active_at": was_active_at,
                        "was_inactive_at": was_inactive_at,
                    },
                    tunnel_list_params.TunnelListParams,
                ),
            ),
            model=cast(Any, TunnelListResponse),  # Union types cannot be passed in as arguments in the type system
        )


class TunnelsResourceWithRawResponse:
    def __init__(self, tunnels: TunnelsResource) -> None:
        self._tunnels = tunnels

        self.list = to_raw_response_wrapper(
            tunnels.list,
        )

    @cached_property
    def cloudflared(self) -> CloudflaredResourceWithRawResponse:
        return CloudflaredResourceWithRawResponse(self._tunnels.cloudflared)

    @cached_property
    def warp_connector(self) -> WARPConnectorResourceWithRawResponse:
        return WARPConnectorResourceWithRawResponse(self._tunnels.warp_connector)


class AsyncTunnelsResourceWithRawResponse:
    def __init__(self, tunnels: AsyncTunnelsResource) -> None:
        self._tunnels = tunnels

        self.list = async_to_raw_response_wrapper(
            tunnels.list,
        )

    @cached_property
    def cloudflared(self) -> AsyncCloudflaredResourceWithRawResponse:
        return AsyncCloudflaredResourceWithRawResponse(self._tunnels.cloudflared)

    @cached_property
    def warp_connector(self) -> AsyncWARPConnectorResourceWithRawResponse:
        return AsyncWARPConnectorResourceWithRawResponse(self._tunnels.warp_connector)


class TunnelsResourceWithStreamingResponse:
    def __init__(self, tunnels: TunnelsResource) -> None:
        self._tunnels = tunnels

        self.list = to_streamed_response_wrapper(
            tunnels.list,
        )

    @cached_property
    def cloudflared(self) -> CloudflaredResourceWithStreamingResponse:
        return CloudflaredResourceWithStreamingResponse(self._tunnels.cloudflared)

    @cached_property
    def warp_connector(self) -> WARPConnectorResourceWithStreamingResponse:
        return WARPConnectorResourceWithStreamingResponse(self._tunnels.warp_connector)


class AsyncTunnelsResourceWithStreamingResponse:
    def __init__(self, tunnels: AsyncTunnelsResource) -> None:
        self._tunnels = tunnels

        self.list = async_to_streamed_response_wrapper(
            tunnels.list,
        )

    @cached_property
    def cloudflared(self) -> AsyncCloudflaredResourceWithStreamingResponse:
        return AsyncCloudflaredResourceWithStreamingResponse(self._tunnels.cloudflared)

    @cached_property
    def warp_connector(self) -> AsyncWARPConnectorResourceWithStreamingResponse:
        return AsyncWARPConnectorResourceWithStreamingResponse(self._tunnels.warp_connector)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .aspa import (
    ASPAResource,
    AsyncASPAResource,
    ASPAResourceWithRawResponse,
    AsyncASPAResourceWithRawResponse,
    ASPAResourceWithStreamingResponse,
    AsyncASPAResourceWithStreamingResponse,
)
from .roas import (
    RoasResource,
    AsyncRoasResource,
    RoasResourceWithRawResponse,
    AsyncRoasResourceWithRawResponse,
    RoasResourceWithStreamingResponse,
    AsyncRoasResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["RPKIResource", "AsyncRPKIResource"]


class RPKIResource(SyncAPIResource):
    @cached_property
    def aspa(self) -> ASPAResource:
        return ASPAResource(self._client)

    @cached_property
    def roas(self) -> RoasResource:
        return RoasResource(self._client)

    @cached_property
    def with_raw_response(self) -> RPKIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RPKIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RPKIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RPKIResourceWithStreamingResponse(self)


class AsyncRPKIResource(AsyncAPIResource):
    @cached_property
    def aspa(self) -> AsyncASPAResource:
        return AsyncASPAResource(self._client)

    @cached_property
    def roas(self) -> AsyncRoasResource:
        return AsyncRoasResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRPKIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRPKIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRPKIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRPKIResourceWithStreamingResponse(self)


class RPKIResourceWithRawResponse:
    def __init__(self, rpki: RPKIResource) -> None:
        self._rpki = rpki

    @cached_property
    def aspa(self) -> ASPAResourceWithRawResponse:
        return ASPAResourceWithRawResponse(self._rpki.aspa)

    @cached_property
    def roas(self) -> RoasResourceWithRawResponse:
        return RoasResourceWithRawResponse(self._rpki.roas)


class AsyncRPKIResourceWithRawResponse:
    def __init__(self, rpki: AsyncRPKIResource) -> None:
        self._rpki = rpki

    @cached_property
    def aspa(self) -> AsyncASPAResourceWithRawResponse:
        return AsyncASPAResourceWithRawResponse(self._rpki.aspa)

    @cached_property
    def roas(self) -> AsyncRoasResourceWithRawResponse:
        return AsyncRoasResourceWithRawResponse(self._rpki.roas)


class RPKIResourceWithStreamingResponse:
    def __init__(self, rpki: RPKIResource) -> None:
        self._rpki = rpki

    @cached_property
    def aspa(self) -> ASPAResourceWithStreamingResponse:
        return ASPAResourceWithStreamingResponse(self._rpki.aspa)

    @cached_property
    def roas(self) -> RoasResourceWithStreamingResponse:
        return RoasResourceWithStreamingResponse(self._rpki.roas)


class AsyncRPKIResourceWithStreamingResponse:
    def __init__(self, rpki: AsyncRPKIResource) -> None:
        self._rpki = rpki

    @cached_property
    def aspa(self) -> AsyncASPAResourceWithStreamingResponse:
        return AsyncASPAResourceWithStreamingResponse(self._rpki.aspa)

    @cached_property
    def roas(self) -> AsyncRoasResourceWithStreamingResponse:
        return AsyncRoasResourceWithStreamingResponse(self._rpki.roas)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel
from .gateway_item import GatewayItem

__all__ = ["ListCreateResponse"]


class ListCreateResponse(BaseModel):
    id: Optional[str] = None
    """Identify the API resource with a UUID."""

    created_at: Optional[datetime] = None

    description: Optional[str] = None
    """Provide the list description."""

    items: Optional[List[GatewayItem]] = None
    """Provide the list items."""

    name: Optional[str] = None
    """Specify the list name."""

    type: Optional[Literal["SERIAL", "URL", "DOMAIN", "EMAIL", "IP", "CATEGORY", "LOCATION", "DEVICE", "AAGUID"]] = None
    """Specify the list type."""

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["DatasetUpdateParams"]


class DatasetUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    case_sensitive: bool
    """Determines if the words should be matched in a case-sensitive manner.

    Only required for custom word lists.
    """

    description: Optional[str]
    """The description of the dataset."""

    name: Optional[str]
    """The name of the dataset, must be unique."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["MeetingGetMeetingParticipantsResponse", "Data", "Paging"]


class Data(BaseModel):
    """Represents a participant."""

    id: str
    """ID of the participant."""

    created_at: datetime
    """When this object was created. The time is returned in ISO format."""

    custom_participant_id: str
    """A unique participant ID generated by the client."""

    preset_name: str
    """Preset applied to the participant."""

    updated_at: datetime
    """When this object was updated. The time is returned in ISO format."""

    name: Optional[str] = None
    """Name of the participant."""

    picture: Optional[str] = None
    """URL to a picture of the participant."""


class Paging(BaseModel):
    end_offset: float

    start_offset: float

    total_count: float


class MeetingGetMeetingParticipantsResponse(BaseModel):
    data: List[Data]

    paging: Paging

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from ..shared_params.token_policy import TokenPolicy
from ..shared.token_condition_cidr_list import TokenConditionCIDRList

__all__ = ["TokenCreateParams", "Condition", "ConditionRequestIP"]


class TokenCreateParams(TypedDict, total=False):
    name: Required[str]
    """Token name."""

    policies: Required[Iterable[TokenPolicy]]
    """List of access policies assigned to the token."""

    condition: Condition

    expires_on: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    The expiration time on or after which the JWT MUST NOT be accepted for
    processing.
    """

    not_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The time before which the token MUST NOT be accepted for processing."""


_ConditionRequestIPReservedKeywords = TypedDict(
    "_ConditionRequestIPReservedKeywords",
    {
        "in": SequenceNotStr[TokenConditionCIDRList],
    },
    total=False,
)


class ConditionRequestIP(_ConditionRequestIPReservedKeywords, total=False):
    """Client IP restrictions."""

    not_in: SequenceNotStr[TokenConditionCIDRList]
    """List of IPv4/IPv6 CIDR addresses."""


class Condition(TypedDict, total=False):
    request_ip: ConditionRequestIP
    """Client IP restrictions."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["SteeringPolicy"]

SteeringPolicy: TypeAlias = Literal[
    "off", "geo", "random", "dynamic_latency", "proximity", "least_outstanding_requests", "least_connections", ""
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dns import (
    DNSResource,
    AsyncDNSResource,
    DNSResourceWithRawResponse,
    AsyncDNSResourceWithRawResponse,
    DNSResourceWithStreamingResponse,
    AsyncDNSResourceWithStreamingResponse,
)
from .subdomains import (
    SubdomainsResource,
    AsyncSubdomainsResource,
    SubdomainsResourceWithRawResponse,
    AsyncSubdomainsResourceWithRawResponse,
    SubdomainsResourceWithStreamingResponse,
    AsyncSubdomainsResourceWithStreamingResponse,
)

__all__ = [
    "DNSResource",
    "AsyncDNSResource",
    "DNSResourceWithRawResponse",
    "AsyncDNSResourceWithRawResponse",
    "DNSResourceWithStreamingResponse",
    "AsyncDNSResourceWithStreamingResponse",
    "SubdomainsResource",
    "AsyncSubdomainsResource",
    "SubdomainsResourceWithRawResponse",
    "AsyncSubdomainsResourceWithRawResponse",
    "SubdomainsResourceWithStreamingResponse",
    "AsyncSubdomainsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .logging import Logging
from ..._models import BaseModel

__all__ = [
    "LogCustomFieldRule",
    "ActionParameters",
    "ActionParametersCookieField",
    "ActionParametersRawResponseField",
    "ActionParametersRequestField",
    "ActionParametersResponseField",
    "ActionParametersTransformedRequestField",
    "ExposedCredentialCheck",
    "Ratelimit",
]


class ActionParametersCookieField(BaseModel):
    """The cookie field to log."""

    name: str
    """The name of the cookie."""


class ActionParametersRawResponseField(BaseModel):
    """The raw response field to log."""

    name: str
    """The name of the response header."""

    preserve_duplicates: Optional[bool] = None
    """Whether to log duplicate values of the same header."""


class ActionParametersRequestField(BaseModel):
    """The raw request field to log."""

    name: str
    """The name of the header."""


class ActionParametersResponseField(BaseModel):
    """The transformed response field to log."""

    name: str
    """The name of the response header."""

    preserve_duplicates: Optional[bool] = None
    """Whether to log duplicate values of the same header."""


class ActionParametersTransformedRequestField(BaseModel):
    """The transformed request field to log."""

    name: str
    """The name of the header."""


class ActionParameters(BaseModel):
    """The parameters configuring the rule's action."""

    cookie_fields: Optional[List[ActionParametersCookieField]] = None
    """The cookie fields to log."""

    raw_response_fields: Optional[List[ActionParametersRawResponseField]] = None
    """The raw response fields to log."""

    request_fields: Optional[List[ActionParametersRequestField]] = None
    """The raw request fields to log."""

    response_fields: Optional[List[ActionParametersResponseField]] = None
    """The transformed response fields to log."""

    transformed_request_fields: Optional[List[ActionParametersTransformedRequestField]] = None
    """The transformed request fields to log."""


class ExposedCredentialCheck(BaseModel):
    """Configuration for exposed credential checking."""

    password_expression: str
    """An expression that selects the password used in the credentials check."""

    username_expression: str
    """An expression that selects the user ID used in the credentials check."""


class Ratelimit(BaseModel):
    """An object configuring the rule's rate limit behavior."""

    characteristics: List[str]
    """
    Characteristics of the request on which the rate limit counter will be
    incremented.
    """

    period: int
    """Period in seconds over which the counter is being incremented."""

    counting_expression: Optional[str] = None
    """An expression that defines when the rate limit counter should be incremented.

    It defaults to the same as the rule's expression.
    """

    mitigation_timeout: Optional[int] = None
    """
    Period of time in seconds after which the action will be disabled following its
    first execution.
    """

    requests_per_period: Optional[int] = None
    """
    The threshold of requests per period after which the action will be executed for
    the first time.
    """

    requests_to_origin: Optional[bool] = None
    """Whether counting is only performed when an origin is reached."""

    score_per_period: Optional[int] = None
    """
    The score threshold per period for which the action will be executed the first
    time.
    """

    score_response_header_name: Optional[str] = None
    """
    A response header name provided by the origin, which contains the score to
    increment rate limit counter with.
    """


class LogCustomFieldRule(BaseModel):
    last_updated: datetime
    """The timestamp of when the rule was last modified."""

    version: str
    """The version of the rule."""

    id: Optional[str] = None
    """The unique ID of the rule."""

    action: Optional[Literal["log_custom_field"]] = None
    """The action to perform when the rule matches."""

    action_parameters: Optional[ActionParameters] = None
    """The parameters configuring the rule's action."""

    categories: Optional[List[str]] = None
    """The categories of the rule."""

    description: Optional[str] = None
    """An informative description of the rule."""

    enabled: Optional[bool] = None
    """Whether the rule should be executed."""

    exposed_credential_check: Optional[ExposedCredentialCheck] = None
    """Configuration for exposed credential checking."""

    expression: Optional[str] = None
    """The expression defining which traffic will match the rule."""

    logging: Optional[Logging] = None
    """An object configuring the rule's logging behavior."""

    ratelimit: Optional[Ratelimit] = None
    """An object configuring the rule's rate limit behavior."""

    ref: Optional[str] = None
    """The reference of the rule (the rule's ID by default)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["MoveCreateResponse"]


class MoveCreateResponse(BaseModel):
    success: bool
    """Whether the operation succeeded"""

    completed_at: Optional[datetime] = None
    """When the move operation completed (UTC)"""

    completed_timestamp: Optional[datetime] = None
    """Deprecated, use `completed_at` instead. End of life: November 1, 2026."""

    destination: Optional[str] = None
    """Destination folder for the message"""

    item_count: Optional[int] = None
    """Number of items moved. End of life: November 1, 2026."""

    message_id: Optional[str] = None
    """Message identifier"""

    operation: Optional[str] = None
    """Type of operation performed"""

    recipient: Optional[str] = None
    """Recipient email address"""

    status: Optional[str] = None
    """Operation status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = ["StreamUpdateResponse", "HTTP", "HTTPCORS", "WorkerBinding", "Format", "FormatJson", "FormatParquet"]


class HTTPCORS(BaseModel):
    """Specifies the CORS options for the HTTP endpoint."""

    origins: Optional[List[str]] = None


class HTTP(BaseModel):
    authentication: bool
    """Indicates that authentication is required for the HTTP endpoint."""

    enabled: bool
    """Indicates that the HTTP endpoint is enabled."""

    cors: Optional[HTTPCORS] = None
    """Specifies the CORS options for the HTTP endpoint."""


class WorkerBinding(BaseModel):
    enabled: bool
    """Indicates that the worker binding is enabled."""


class FormatJson(BaseModel):
    type: Literal["json"]

    decimal_encoding: Optional[Literal["number", "string", "bytes"]] = None

    timestamp_format: Optional[Literal["rfc3339", "unix_millis"]] = None

    unstructured: Optional[bool] = None


class FormatParquet(BaseModel):
    type: Literal["parquet"]

    compression: Optional[Literal["uncompressed", "snappy", "gzip", "zstd", "lz4"]] = None

    row_group_bytes: Optional[int] = None


Format: TypeAlias = Annotated[Union[FormatJson, FormatParquet], PropertyInfo(discriminator="type")]


class StreamUpdateResponse(BaseModel):
    id: str
    """Indicates a unique identifier for this stream."""

    created_at: datetime

    http: HTTP

    modified_at: datetime

    name: str
    """Indicates the name of the Stream."""

    version: int
    """Indicates the current version of this stream."""

    worker_binding: WorkerBinding

    endpoint: Optional[str] = None
    """Indicates the endpoint URL of this stream."""

    format: Optional[Format] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, strip_not_given, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from .prebuilt_policies import (
    PrebuiltPoliciesResource,
    AsyncPrebuiltPoliciesResource,
    PrebuiltPoliciesResourceWithRawResponse,
    AsyncPrebuiltPoliciesResourceWithRawResponse,
    PrebuiltPoliciesResourceWithStreamingResponse,
    AsyncPrebuiltPoliciesResourceWithStreamingResponse,
)
from ....types.magic_cloud_networking import (
    catalog_sync_edit_params,
    catalog_sync_create_params,
    catalog_sync_delete_params,
    catalog_sync_update_params,
)
from ....types.magic_cloud_networking.catalog_sync_get_response import CatalogSyncGetResponse
from ....types.magic_cloud_networking.catalog_sync_edit_response import CatalogSyncEditResponse
from ....types.magic_cloud_networking.catalog_sync_list_response import CatalogSyncListResponse
from ....types.magic_cloud_networking.catalog_sync_create_response import CatalogSyncCreateResponse
from ....types.magic_cloud_networking.catalog_sync_delete_response import CatalogSyncDeleteResponse
from ....types.magic_cloud_networking.catalog_sync_update_response import CatalogSyncUpdateResponse
from ....types.magic_cloud_networking.catalog_sync_refresh_response import CatalogSyncRefreshResponse

__all__ = ["CatalogSyncsResource", "AsyncCatalogSyncsResource"]


class CatalogSyncsResource(SyncAPIResource):
    @cached_property
    def prebuilt_policies(self) -> PrebuiltPoliciesResource:
        return PrebuiltPoliciesResource(self._client)

    @cached_property
    def with_raw_response(self) -> CatalogSyncsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CatalogSyncsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CatalogSyncsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CatalogSyncsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        destination_type: Literal["NONE", "ZERO_TRUST_LIST"],
        name: str,
        update_mode: Literal["AUTO", "MANUAL"],
        description: str | Omit = omit,
        policy: str | Omit = omit,
        forwarded: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncCreateResponse:
        """
        Create a new Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {**strip_not_given({"forwarded": forwarded}), **(extra_headers or {})}
        return self._post(
            path_template("/accounts/{account_id}/magic/cloud/catalog-syncs", account_id=account_id),
            body=maybe_transform(
                {
                    "destination_type": destination_type,
                    "name": name,
                    "update_mode": update_mode,
                    "description": description,
                    "policy": policy,
                },
                catalog_sync_create_params.CatalogSyncCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncCreateResponse], ResultWrapper[CatalogSyncCreateResponse]),
        )

    def update(
        self,
        sync_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        policy: str | Omit = omit,
        update_mode: Literal["AUTO", "MANUAL"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncUpdateResponse:
        """
        Update a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "policy": policy,
                    "update_mode": update_mode,
                },
                catalog_sync_update_params.CatalogSyncUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncUpdateResponse], ResultWrapper[CatalogSyncUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CatalogSyncListResponse]:
        """
        List Catalog Syncs (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/catalog-syncs", account_id=account_id),
            page=SyncSinglePage[CatalogSyncListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CatalogSyncListResponse,
        )

    def delete(
        self,
        sync_id: str,
        *,
        account_id: str,
        delete_destination: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncDeleteResponse:
        """
        Delete a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"delete_destination": delete_destination}, catalog_sync_delete_params.CatalogSyncDeleteParams
                ),
                post_parser=ResultWrapper[CatalogSyncDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncDeleteResponse], ResultWrapper[CatalogSyncDeleteResponse]),
        )

    def edit(
        self,
        sync_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        policy: str | Omit = omit,
        update_mode: Literal["AUTO", "MANUAL"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncEditResponse:
        """
        Update a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "policy": policy,
                    "update_mode": update_mode,
                },
                catalog_sync_edit_params.CatalogSyncEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncEditResponse], ResultWrapper[CatalogSyncEditResponse]),
        )

    def get(
        self,
        sync_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncGetResponse:
        """
        Read a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncGetResponse], ResultWrapper[CatalogSyncGetResponse]),
        )

    def refresh(
        self,
        sync_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Refresh a Catalog Sync's destination by running the sync policy against latest
        resource catalog (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}/refresh",
                account_id=account_id,
                sync_id=sync_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncRefreshResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class AsyncCatalogSyncsResource(AsyncAPIResource):
    @cached_property
    def prebuilt_policies(self) -> AsyncPrebuiltPoliciesResource:
        return AsyncPrebuiltPoliciesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCatalogSyncsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCatalogSyncsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCatalogSyncsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCatalogSyncsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        destination_type: Literal["NONE", "ZERO_TRUST_LIST"],
        name: str,
        update_mode: Literal["AUTO", "MANUAL"],
        description: str | Omit = omit,
        policy: str | Omit = omit,
        forwarded: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncCreateResponse:
        """
        Create a new Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {**strip_not_given({"forwarded": forwarded}), **(extra_headers or {})}
        return await self._post(
            path_template("/accounts/{account_id}/magic/cloud/catalog-syncs", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "destination_type": destination_type,
                    "name": name,
                    "update_mode": update_mode,
                    "description": description,
                    "policy": policy,
                },
                catalog_sync_create_params.CatalogSyncCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncCreateResponse], ResultWrapper[CatalogSyncCreateResponse]),
        )

    async def update(
        self,
        sync_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        policy: str | Omit = omit,
        update_mode: Literal["AUTO", "MANUAL"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncUpdateResponse:
        """
        Update a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "policy": policy,
                    "update_mode": update_mode,
                },
                catalog_sync_update_params.CatalogSyncUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncUpdateResponse], ResultWrapper[CatalogSyncUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CatalogSyncListResponse, AsyncSinglePage[CatalogSyncListResponse]]:
        """
        List Catalog Syncs (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/catalog-syncs", account_id=account_id),
            page=AsyncSinglePage[CatalogSyncListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CatalogSyncListResponse,
        )

    async def delete(
        self,
        sync_id: str,
        *,
        account_id: str,
        delete_destination: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncDeleteResponse:
        """
        Delete a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"delete_destination": delete_destination}, catalog_sync_delete_params.CatalogSyncDeleteParams
                ),
                post_parser=ResultWrapper[CatalogSyncDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncDeleteResponse], ResultWrapper[CatalogSyncDeleteResponse]),
        )

    async def edit(
        self,
        sync_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        name: str | Omit = omit,
        policy: str | Omit = omit,
        update_mode: Literal["AUTO", "MANUAL"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncEditResponse:
        """
        Update a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "policy": policy,
                    "update_mode": update_mode,
                },
                catalog_sync_edit_params.CatalogSyncEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncEditResponse], ResultWrapper[CatalogSyncEditResponse]),
        )

    async def get(
        self,
        sync_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CatalogSyncGetResponse:
        """
        Read a Catalog Sync (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}", account_id=account_id, sync_id=sync_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CatalogSyncGetResponse], ResultWrapper[CatalogSyncGetResponse]),
        )

    async def refresh(
        self,
        sync_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """
        Refresh a Catalog Sync's destination by running the sync policy against latest
        resource catalog (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not sync_id:
            raise ValueError(f"Expected a non-empty value for `sync_id` but received {sync_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/magic/cloud/catalog-syncs/{sync_id}/refresh",
                account_id=account_id,
                sync_id=sync_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CatalogSyncRefreshResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )


class CatalogSyncsResourceWithRawResponse:
    def __init__(self, catalog_syncs: CatalogSyncsResource) -> None:
        self._catalog_syncs = catalog_syncs

        self.create = to_raw_response_wrapper(
            catalog_syncs.create,
        )
        self.update = to_raw_response_wrapper(
            catalog_syncs.update,
        )
        self.list = to_raw_response_wrapper(
            catalog_syncs.list,
        )
        self.delete = to_raw_response_wrapper(
            catalog_syncs.delete,
        )
        self.edit = to_raw_response_wrapper(
            catalog_syncs.edit,
        )
        self.get = to_raw_response_wrapper(
            catalog_syncs.get,
        )
        self.refresh = to_raw_response_wrapper(
            catalog_syncs.refresh,
        )

    @cached_property
    def prebuilt_policies(self) -> PrebuiltPoliciesResourceWithRawResponse:
        return PrebuiltPoliciesResourceWithRawResponse(self._catalog_syncs.prebuilt_policies)


class AsyncCatalogSyncsResourceWithRawResponse:
    def __init__(self, catalog_syncs: AsyncCatalogSyncsResource) -> None:
        self._catalog_syncs = catalog_syncs

        self.create = async_to_raw_response_wrapper(
            catalog_syncs.create,
        )
        self.update = async_to_raw_response_wrapper(
            catalog_syncs.update,
        )
        self.list = async_to_raw_response_wrapper(
            catalog_syncs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            catalog_syncs.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            catalog_syncs.edit,
        )
        self.get = async_to_raw_response_wrapper(
            catalog_syncs.get,
        )
        self.refresh = async_to_raw_response_wrapper(
            catalog_syncs.refresh,
        )

    @cached_property
    def prebuilt_policies(self) -> AsyncPrebuiltPoliciesResourceWithRawResponse:
        return AsyncPrebuiltPoliciesResourceWithRawResponse(self._catalog_syncs.prebuilt_policies)


class CatalogSyncsResourceWithStreamingResponse:
    def __init__(self, catalog_syncs: CatalogSyncsResource) -> None:
        self._catalog_syncs = catalog_syncs

        self.create = to_streamed_response_wrapper(
            catalog_syncs.create,
        )
        self.update = to_streamed_response_wrapper(
            catalog_syncs.update,
        )
        self.list = to_streamed_response_wrapper(
            catalog_syncs.list,
        )
        self.delete = to_streamed_response_wrapper(
            catalog_syncs.delete,
        )
        self.edit = to_streamed_response_wrapper(
            catalog_syncs.edit,
        )
        self.get = to_streamed_response_wrapper(
            catalog_syncs.get,
        )
        self.refresh = to_streamed_response_wrapper(
            catalog_syncs.refresh,
        )

    @cached_property
    def prebuilt_policies(self) -> PrebuiltPoliciesResourceWithStreamingResponse:
        return PrebuiltPoliciesResourceWithStreamingResponse(self._catalog_syncs.prebuilt_policies)


class AsyncCatalogSyncsResourceWithStreamingResponse:
    def __init__(self, catalog_syncs: AsyncCatalogSyncsResource) -> None:
        self._catalog_syncs = catalog_syncs

        self.create = async_to_streamed_response_wrapper(
            catalog_syncs.create,
        )
        self.update = async_to_streamed_response_wrapper(
            catalog_syncs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            catalog_syncs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            catalog_syncs.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            catalog_syncs.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            catalog_syncs.get,
        )
        self.refresh = async_to_streamed_response_wrapper(
            catalog_syncs.refresh,
        )

    @cached_property
    def prebuilt_policies(self) -> AsyncPrebuiltPoliciesResourceWithStreamingResponse:
        return AsyncPrebuiltPoliciesResourceWithStreamingResponse(self._catalog_syncs.prebuilt_policies)
